'
' Copyright (c) 2015, MindFusion LLC - Bulgaria.
'

Imports System.Drawing.Drawing2D


Public Class MainForm

    Sub New()

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.

        For i As Integer = 15 To 345
            Dim rng As Range = New Range()
            rng.MinValue = i
            rng.MaxValue = i + 15
            rng.Fill = New MindFusion.Drawing.SolidBrush( _
                Color.FromArgb(255, 64, 64, 64))
            rng.Stroke = New MindFusion.Drawing.Pen( _
                Color.FromArgb(255, 32, 32, 32))
            rng.Alignment = Alignment.TrueCenter
            compass.Scales(1).Ranges.Add(rng)
            i += 29
        Next i

        AddHandler compass.Scales(1).QueryLabelValue, AddressOf InnerScale_QueryLabelValue

    End Sub

    Sub InnerScale_QueryLabelValue(ByVal sender As Object, ByVal e As QueryLabelValueEventArgs)
        If e.CalculatedLabelValue = 0 Then
            e.NewValue = "N"
        ElseIf e.CalculatedLabelValue = 45 Then
            e.NewValue = "NE"
        ElseIf e.CalculatedLabelValue = 90 Then
            e.NewValue = "E"
        ElseIf e.CalculatedLabelValue = 135 Then
            e.NewValue = "SE"
        ElseIf e.CalculatedLabelValue = 180 Then
            e.NewValue = "S"
        ElseIf e.CalculatedLabelValue = 225 Then
            e.NewValue = "SW"
        ElseIf e.CalculatedLabelValue = 270 Then
            e.NewValue = "W"
        ElseIf e.CalculatedLabelValue = 315 Then
            e.NewValue = "NW"
        End If
    End Sub

    Private Sub compass_PaintPointer(ByVal sender As Object, ByVal e As CustomPaintEventArgs) Handles compass.PaintPointer
        If e.Element.Name = "MainPointer" Then
            Dim state As GraphicsState = e.Graphics.Save()
            e.Graphics.ScaleTransform(e.Element.RenderSize.Width, e.Element.RenderSize.Height)

            Dim boundingRect As RectangleF = New RectangleF(0, 0, 1, 1)

            Dim commonStroke As Pen = New Pen(Color.FromArgb(255, 32, 32, 32), 0.015F)

            Dim fill1path As GraphicsPath = New GraphicsPath()

            Dim pt1 As PointF = New PointF(0.5F, 0)
            Dim pt2 As PointF = New PointF(1, 0.5F)
            Dim pt3 As PointF = New PointF(0.5F, 1)

            fill1path.AddPolygon(New PointF() {pt1, pt2, pt3})

            Dim fill1 As PathGradientBrush = New PathGradientBrush(fill1path)

            fill1.CenterPoint = New PointF(0.5F, 0)

            Dim fill1blend As ColorBlend = New ColorBlend()
            fill1blend.Positions = New Single() {0, 0.3F, 1}
            fill1blend.Colors = New Color() { _
                Color.FromArgb(255, 193, 0, 0), _
                Color.FromArgb(255, 255, 0, 0), _
                Color.FromArgb(255, 102, 0, 0)}


            fill1.InterpolationColors = fill1blend

            e.Graphics.FillPolygon(fill1, New PointF() {pt1, pt2, pt3})

            Dim pt4 As PointF = New PointF(0, 0.5F)

            fill1path.AddPolygon(New PointF() {pt1, pt4, pt3})

            Dim fill2 As PathGradientBrush = New PathGradientBrush(fill1path)

            fill2.CenterPoint = New PointF(0.5F, 0)
            Dim fill2blend As ColorBlend = New ColorBlend()
            fill2blend.Positions = New Single() {0, 0.3F, 1}
            fill2blend.Colors = New Color() { _
                Color.FromArgb(255, 193, 193, 193), _
                Color.FromArgb(255, 204, 204, 204), _
                Color.FromArgb(255, 102, 102, 102)}

            fill2.InterpolationColors = fill2blend

            e.Graphics.FillPolygon(fill2, New PointF() {pt1, pt4, pt3})

            e.Graphics.DrawPolygon(commonStroke, New PointF() {pt1, pt2, pt3, pt4})

            Dim center As PointF = New PointF(boundingRect.Width / 2, boundingRect.Height / 2)
            Dim sizeF As SizeF = New SizeF(0.03F, 0.3F)

            e.Graphics.DrawEllipse(commonStroke, New RectangleF(New PointF(center.X - sizeF.Width / 2.0F, center.Y - sizeF.Height / 2.0F), sizeF))
            e.Graphics.FillEllipse(New SolidBrush(Color.DimGray), New RectangleF(New PointF(center.X - sizeF.Width / 2.0F, center.Y - sizeF.Height / 2.0F), sizeF))

            fill1.Dispose()
            fill2.Dispose()
            fill1path.Dispose()
            e.Graphics.Restore(state)
        End If
    End Sub

    Private Sub compass_PrepaintBackground(ByVal sender As Object, ByVal e As PrepaintEventArgs) Handles compass.PrepaintBackground
        ' Cancels the painting of the default background
        e.CancelDefaultPainting = True

        Dim boundingRect As RectangleF = New RectangleF(0, 0, e.Element.RenderSize.Width, e.Element.RenderSize.Height)

        'Background layer 1
        Dim fill1 As LinearGradientBrush = New LinearGradientBrush( _
            New Point(0, 0), New Point(boundingRect.Right, boundingRect.Bottom), _
            Color.FromArgb(255, 148, 148, 148), _
            Color.FromArgb(255, 48, 48, 48))

        Dim blend As Blend = New Blend()
        blend.Positions = New Single() {0, 0.2F, 0.8F, 1}
        blend.Factors = New Single() {0, 0, 1, 1}

        fill1.Blend = blend
        e.Graphics.FillEllipse(fill1, boundingRect)

        fill1.Dispose()

        boundingRect.Inflate(-0.013F * boundingRect.Width, -0.013F * boundingRect.Height)

        'Background layer 2
        Dim fill2 As LinearGradientBrush = New LinearGradientBrush( _
            New Point(boundingRect.Right, boundingRect.Bottom), New Point(0, 0), _
            Color.FromArgb(255, 148, 148, 148), _
            Color.FromArgb(255, 48, 48, 48))

        fill2.Blend = blend
        e.Graphics.FillEllipse(fill2, boundingRect)

        fill2.Dispose()

        boundingRect.Inflate(-0.013F * boundingRect.Width, -0.013F * boundingRect.Height)

        'Background layer 3
        Dim fill3 As Brush = New SolidBrush( _
                Color.FromArgb(255, 222, 222, 222))
        e.Graphics.FillEllipse(fill3, boundingRect)

        fill3.Dispose()
    End Sub

    Private Sub compass_PrepaintForeground(ByVal sender As Object, ByVal e As PrepaintEventArgs) Handles compass.PrepaintForeground
        ' Cancels the painting of the default foreground
        e.CancelDefaultPainting = True

        Dim boundingRect As RectangleF = New RectangleF(0, 0, 1, 1)

        Dim areaPath As GraphicsPath = New GraphicsPath()
        areaPath.AddEllipse(boundingRect)

        Dim area1Fill As MindFusion.Drawing.PathGradientBrush = _
            New MindFusion.Drawing.PathGradientBrush( _
            Color.FromArgb(96, Color.White), Color.Transparent)
        area1Fill.Path = areaPath

        Dim area2Fill As MindFusion.Drawing.PathGradientBrush = _
            New MindFusion.Drawing.PathGradientBrush( _
            Color.FromArgb(64, Color.White), Color.Transparent)
        area2Fill.Path = areaPath

        Dim area1 As ArcArea = New ArcArea()
        area1.StartAngle = 160
        area1.EndAngle = -20
        area1.Margin = New Thickness(0.03F)
        area1.Fill = area1Fill

        Dim area2 As ArcArea = New ArcArea()
        area2.StartAngle = 140
        area2.EndAngle = -40
        area2.Margin = New Thickness(0.03F)
        area2.Fill = area2Fill

        e.PaintVisualElement(area1, e.Element.RenderSize)
        e.PaintVisualElement(area2, e.Element.RenderSize)

        areaPath.Dispose()
    End Sub

    Private Sub compass_PrepaintScale(ByVal sender As Object, ByVal e As PrepaintEventArgs) Handles compass.PrepaintScale
        If e.Element.Name = "InnerScale" Then

            Dim state As GraphicsState = e.Graphics.Save()
            e.Graphics.ScaleTransform(e.Element.RenderSize.Width, e.Element.RenderSize.Height)

            Dim commonStroke As Pen = New Pen(Color.FromArgb(255, 32, 32, 32), 0.001F)
            Dim lightFill As Brush = New SolidBrush(Color.FromArgb(255, 191, 191, 191))
            Dim darkFill As Brush = New SolidBrush(Color.FromArgb(255, 128, 128, 128))

            ' Four main ploygons of the "sun"

            ' Polygon 1
            Dim path1 As GraphicsPath = New GraphicsPath()

            Dim pt1 As PointF = New PointF(0.44F, 0.5F)
            Dim pt2 As PointF = New PointF(0.5F, 0.15F)
            Dim pt3 As PointF = New PointF(0.5F, 0.85F)
            Dim pt4 As PointF = New PointF(0.56F, 0.5F)

            path1.AddPolygon(New PointF() {pt1, pt2, pt3, pt4})

            e.Graphics.FillPolygon(lightFill, New PointF() {pt1, pt2, pt3, pt4})

            Dim rotateMatrix As Matrix = New Matrix()
            rotateMatrix.RotateAt(90, New PointF(0.5F, 0.5F))

            ' Polygon 2
            path1.Transform(rotateMatrix)
            e.Graphics.FillPath(lightFill, path1)

            path1.Dispose()

            ' Polygon 3
            Dim path2 As GraphicsPath = New GraphicsPath()

            path2.AddPolygon(New PointF() {pt4, pt2, pt3, pt1})
            e.Graphics.FillPolygon(darkFill, New PointF() {pt4, pt2, pt3, pt1})

            ' Polygon 4
            path2.Transform(rotateMatrix)
            e.Graphics.FillPath(darkFill, path2)

            path2.Dispose()

            Dim pt5 As PointF = New PointF(0.5F, 0.56F)
            Dim pt6 As PointF = New PointF(0.15F, 0.5F)
            Dim pt7 As PointF = New PointF(0.5F, 0.44F)
            Dim pt8 As PointF = New PointF(0.85F, 0.5F)

            e.Graphics.DrawPolygon(commonStroke, New PointF() {pt1, pt2, pt4, pt3})
            e.Graphics.DrawPolygon(commonStroke, New PointF() {pt5, pt6, pt7, pt8})


            ' Four minor rays of the "sun"
            Dim path3 As GraphicsPath = New GraphicsPath()

            Dim p1 As PointF = New PointF(0.46F, 0.5F)
            Dim p2 As PointF = New PointF(0.3F, 0.3F)
            Dim p3 As PointF = New PointF(0.5F, 0.46F)
            Dim p4 As PointF = New PointF(0.7F, 0.3F)
            Dim p5 As PointF = New PointF(0.53F, 0.5F)
            Dim p6 As PointF = New PointF(0.7F, 0.7F)
            Dim p7 As PointF = New PointF(0.5F, 0.53F)
            Dim p8 As PointF = New PointF(0.3F, 0.7F)

            path3.AddPolygon(New PointF() {p1, p2, p3, p4, p5, p6, p7, p8})
            e.Graphics.FillPolygon(lightFill, _
                New PointF() {p1, p2, p3, p4, p5, p6, p7, p8})

            e.Graphics.DrawPolygon(commonStroke, _
                New PointF() {p1, p2, p3, p4, p5, p6, p7, p8})

            path3.Dispose()

            ' Center of the "sun"
            Dim boundingRect As RectangleF = New RectangleF(0, 0, 1, 1)

            boundingRect.Inflate(-0.37F, -0.37F)

            e.Graphics.DrawEllipse(commonStroke, boundingRect)
            e.Graphics.FillEllipse(lightFill, boundingRect)

            boundingRect.Inflate(-0.01F, -0.01F)

            e.Graphics.DrawEllipse(commonStroke, boundingRect)
            e.Graphics.FillEllipse( _
                New SolidBrush(Color.FromArgb(255, 222, 222, 222)), boundingRect)

            commonStroke.Dispose()
            lightFill.Dispose()
            darkFill.Dispose()
            rotateMatrix.Dispose()
            e.Graphics.Restore(state)
        End If
    End Sub
End Class
